using System;
using System.Collections.Generic;
using System.Text;
using CodeSmith.Engine;
using SchemaExplorer;
using System.Data;
using System.ComponentModel;
using System.IO;
namespace YcCodeHelper
{
    public class CommonScriptCode : CodeTemplate
    {
        #region 
        private TableSchema _sourceTable;

        [Category("02.ݱ")]
        [Description("ɴƵݱ")]
        public TableSchema SourceTable
        {
            get { return _sourceTable; }
            set { _sourceTable = value; }
        }

        private string _projectName;

        [Category("01.Ŀ")]
        [Description("Ŀƣռ")]
        public string ProjectName
        {
            get { return _projectName; }
            set { _projectName = value; }
        }
        
        #endregion

        

        #region  nettiers2.2Ľű
        public string GetSqlParameterStatement(ColumnSchema column)
        {
            string param = "@" + column.Name + " " + column.NativeType;

            switch (column.DataType)
            {
                case DbType.Decimal:
                    {
                        param += "(" + column.Precision + ", " + column.Scale + ")";
                        break;
                    }
                default:
                    {
                        if (column.Size > 0)
                        {
                            param += "(" + column.Size + ")";
                        }
                        break;
                    }
            }

            return param;
        }
        public string GetCSharpVariableType(ColumnSchema column, string str)
        {
            if (column.Name.EndsWith("TypeCode")) return str;

            switch (column.DataType)
            {
                case DbType.AnsiString: return str + ".ToString()";
                case DbType.AnsiStringFixedLength: return str + ".ToString()";
                case DbType.Binary: return "((" + str + ")==DBNull.Value) ? new byte[0] : (Byte[])(" + str + ")";
                case DbType.Boolean: return "(bool)" + str;
                case DbType.Byte: return "((" + str + ")==DBNull.Value)?Convert.ToByte(0):Convert.ToByte(" + str + ")";
                case DbType.Currency: return "((" + str + ")==DBNull.Value)?0:Convert.ToDecimal(" + str + ")";
                case DbType.Date: return "((" + str + ")==DBNull.Value)?" + DateTime.Now + ":Convert.ToDateTime(" + str + ")";
                case DbType.DateTime: return "((" + str + ")==DBNull.Value)?Convert.ToDateTime(1900-1-1):Convert.ToDateTime(" + str + ")";
                case DbType.Decimal: return "((" + str + ")==DBNull.Value)?0:Convert.ToDecimal(" + str + ")";
                case DbType.Double: return "((" + str + ")==DBNull.Value)?0:Convert.ToDouble(" + str + ")";
                case DbType.Guid: return "(Guid)" + str;
                case DbType.Int16: return "(short)" + str;
                case DbType.Int32: return "((" + str + ")==DBNull.Value)?0:Convert.ToInt32(" + str + ")";
                case DbType.Int64: return "(long)" + str;
                case DbType.Object: return "(object)" + str;
                case DbType.SByte: return "Convert.ToSByte(" + str + ")";
                case DbType.Single: return "(float)" + str;
                case DbType.String: return str + ".ToString()";
                case DbType.StringFixedLength: return str + ".ToString()";
                case DbType.Time: return "(DateTime)" + str;
                case DbType.UInt16: return "(ushort)" + str;
                case DbType.UInt32: return "(uint)" + str;
                case DbType.UInt64: return "(ulong)" + str;
                case DbType.VarNumeric: return "Convert.ToDecimal(" + str + ")";
                default:
                    {
                        return "__UNKNOWN__" + str;
                    }
            }
        }
        public string GetCSharpSqlType(ColumnSchema column)
        {
            if (column.Name.EndsWith("TypeCode")) return "";

            switch (column.DataType)
            {
                case DbType.AnsiString: return ".VarChar";
                case DbType.AnsiStringFixedLength: return ".VarChar";
                case DbType.Binary: return ".Binary";
                case DbType.Boolean: return ".Bit";
                case DbType.Byte: return ".TinyInt";
                case DbType.Currency: return "";
                case DbType.Date: return "";
                case DbType.DateTime: return ".DateTime";
                case DbType.Decimal: return ".Decimal";
                case DbType.Double: return ".Float";
                case DbType.Guid: return ".UniqueIdentifier";
                case DbType.Int16: return ".SmallInt";
                case DbType.Int32: return ".Int";
                case DbType.Int64: return ".BigInt";
                case DbType.Object: return "";
                case DbType.SByte: return "";
                case DbType.Single: return ".Float";
                case DbType.String: return ".VarChar";
                case DbType.StringFixedLength: return ".VarChar";
                case DbType.Time: return ".DateTime";
                case DbType.UInt16: return ".";
                case DbType.UInt32: return "";
                case DbType.UInt64: return "";
                case DbType.VarNumeric: return "";

                default:
                    {
                        return "__UNKNOWN__";
                    }
            }
        }


        public int GetColumnPosition(string name)
        {
            int i = 0;
            foreach (ColumnSchema column in SourceTable.Columns)
            {
                if (column.Name == name)
                {
                    return i;
                }
                i++;
            }
            return i;
        }

        public string GetMemberVariableDeclarationStatement(ColumnSchema column)
        {
            return GetMemberVariableDeclarationStatement("private", column);
        }

        public string GetMemberVariableDeclarationStatement(string protectionLevel, ColumnSchema column)
        {
            string statement = protectionLevel + " ";
            statement += GetCSharpVariableType(column) + " " + GetMemberVariableName(column);

            string defaultValue = GetMemberVariableDefaultValue(column);
            if (defaultValue != "")
            {
                statement += " = " + defaultValue;
            }

            statement += ";";

            return statement;
        }

        public string GetReaderAssignmentStatement(ColumnSchema column, int index)
        {
            string statement = "if (!reader.IsDBNull(" + index.ToString() + ")) ";
            statement += GetMemberVariableName(column) + " = ";

            if (column.Name.EndsWith("TypeCode")) statement += "(" + column.Name + ")";

            statement += "reader." + GetReaderMethod(column) + "(" + index.ToString() + ");";

            return statement;
        }

        public string GetCamelCaseName(string value)
        {
            return value.Substring(0, 1).ToLower() + value.Substring(1);
        }

        public string ConvertFirstCharToUpper(string value)
        {
            return value.Substring(0, 1).ToUpper() + value.Substring(1);
        }

        public string GetMemberVariableName(ColumnSchema column)
        {
            string propertyName = GetPropertyName(column);
            string memberVariableName = GetCamelCaseName(propertyName);

            return memberVariableName;
        }

        public string GetPropertyName(ColumnSchema column)
        {
            string propertyName = ConvertFirstCharToUpper(column.Name);

            if (propertyName == column.Table.Name + "Name") return "Name";
            if (propertyName == column.Table.Name + "Description") return "Description";

            if (propertyName.EndsWith("TypeCode")) propertyName = propertyName.Substring(0, propertyName.Length - 4);

            return propertyName;
        }

        public string GetMemberVariableDefaultValue(ColumnSchema column)
        {
            string result = column.ExtendedProperties["CS_Default"].Value.ToString();
            string n = string.Empty;
            foreach (char c in result)
            {
                switch (c)
                {
                    case '(':
                    case ')':
                    case '\'':
                        break;
                    default:
                        n = n + c;
                        break;
                }

            }
            result = "\"" + n + "\"";
            switch (column.DataType)
            {
                case DbType.Guid:
                    {
                        return "Guid.Empty";
                    }
                case DbType.AnsiString:
                case DbType.AnsiStringFixedLength:
                case DbType.String:
                case DbType.StringFixedLength:
                    {
                        return result;                        
                    }
                case DbType.Int16:
                case DbType.Int32:
                case DbType.Int64:
                case DbType.Single:
                    {
                        return "0";
                    }
                case DbType.DateTime:
                    {
                        return "DateTime.Parse(" +  result + ")";
                    }
                case DbType.Date:
                    {
                        return "DateTime.Parse(" +  result  + ")";
                    }
                case DbType.Boolean:
                    {
                        return "false";
                    }
                default:
                    {
                        return "";
                    }
            }
        }

        
        public string GetCSharpVariableType(ColumnSchema column)
        {
            if (column.Name.EndsWith("TypeCode")) return column.Name;

            switch (column.DataType)
            {
                case DbType.AnsiString: return "string";
                case DbType.AnsiStringFixedLength: return "string";
                case DbType.Binary: return "byte[]";
                case DbType.Boolean: return "bool";
                case DbType.Byte: return "byte";
                case DbType.Currency: return "decimal";
                case DbType.Date: return "DateTime";
                case DbType.DateTime: return "DateTime";
                case DbType.Decimal: return "decimal";
                case DbType.Double: return "double";
                case DbType.Guid: return "Guid";
                case DbType.Int16: return "short";
                case DbType.Int32: return "int";
                case DbType.Int64: return "long";
                case DbType.Object: return "object";
                case DbType.SByte: return "sbyte";
                case DbType.Single: return "float";
                case DbType.String: return "string";
                case DbType.StringFixedLength: return "string";
                case DbType.Time: return "TimeSpan";
                case DbType.UInt16: return "ushort";
                case DbType.UInt32: return "uint";
                case DbType.UInt64: return "ulong";
                case DbType.VarNumeric: return "decimal";
                default:
                    {
                        return "__UNKNOWN__" + column.NativeType;
                    }
            }
        }

        public string GetReaderMethod(ColumnSchema column)
        {
            switch (column.DataType)
            {
                case DbType.Byte:
                    {
                        return "GetByte";
                    }
                case DbType.Int16:
                    {
                        return "GetInt16";
                    }
                case DbType.Int32:
                    {
                        return "GetInt32";
                    }
                case DbType.Int64:
                    {
                        return "GetInt64";
                    }
                case DbType.AnsiStringFixedLength:
                case DbType.AnsiString:
                case DbType.String:
                case DbType.StringFixedLength:
                    {
                        return "GetString";
                    }
                case DbType.Boolean:
                    {
                        return "GetBoolean";
                    }
                case DbType.Guid:
                    {
                        return "GetGuid";
                    }
                case DbType.Currency:
                case DbType.Decimal:
                    {
                        return "GetDecimal";
                    }
                case DbType.DateTime:
                case DbType.Date:
                    {
                        return "GetDateTime";
                    }
                default:
                    {
                        return "__SQL__" + column.DataType;
                    }
            }
        }

        public string GetClassName(TableSchema table)
        {
            /*
            if (table.Name.EndsWith("s"))
            {
                return table.Name.Substring(0, table.Name.Length - 1);
            }
            else
            {
                return table.Name;
            }
            */
            if (table == null)
            {
                return null;
            }
            return table.Name;
        }

        public string GetSqlDbType(ColumnSchema column)
        {
            switch (column.NativeType)
            {
                case "bigint": return "BigInt";
                case "binary": return "Binary";
                case "bit": return "Bit";
                case "char": return "Char";
                case "datetime": return "DateTime";
                case "decimal": return "Decimal";
                case "float": return "Float";
                case "image": return "Image";
                case "int": return "Int";
                case "money": return "Money";
                case "nchar": return "NChar";
                case "ntext": return "NText";
                case "numeric": return "Decimal";
                case "nvarchar": return "NVarChar";
                case "real": return "Real";
                case "smalldatetime": return "SmallDateTime";
                case "smallint": return "SmallInt";
                case "smallmoney": return "SmallMoney";
                case "sql_variant": return "Variant";
                case "sysname": return "NChar";
                case "text": return "Text";
                case "timestamp": return "Timestamp";
                case "tinyint": return "TinyInt";
                case "uniqueidentifier": return "UniqueIdentifier";
                case "varbinary": return "VarBinary";
                case "varchar": return "VarChar";
                default: return "__UNKNOWN__" + column.NativeType;
            }
        }

        public string GetPrimaryKeyVariableExpression(TableSchema table)
        {
            return GetCSharpVariableType(table.PrimaryKey.MemberColumns[0]) + " " + GetCamelCaseName(table.PrimaryKey.MemberColumns[0].Name);
        }

        public string GetPrimaryKeyType(TableSchema table)
        {
            if (table.PrimaryKey.MemberColumns.Count == 1)
            {
                return GetCSharpVariableType(table.PrimaryKey.MemberColumns[0]);
            }
            else
            {
                throw new ApplicationException("This template will not work on primary keys with more than one member column.");
            }
        }

        public string GetPopulationConstructorParams(SchemaExplorer.TableSchema table)
        {
            System.Text.StringBuilder sb = new StringBuilder();
            foreach (ColumnSchema column in table.Columns)
            {
                if (sb.Length != 0)
                    sb.Append(", ");

                sb.Append(GetCSharpVariableType(column));
                sb.Append(" ");
                sb.Append(GetCamelCaseName(column.Name));
            }
            return sb.ToString();
        }

        public string GetCollectionClassName()
        {
            return GetClassName(SourceTable) + "Collection";
        }

        public string GetLowerCaseName(string value)
        {
            return value.ToLower();
        }

        public string GetModuleInstanceName(TableSchema table)
        {
            return "_" + table.Name + "Model";
        }

        public string GetAddParametersString(ColumnSchema column)
        {
            DbType dbtype = column.DataType;
            string result = String.Empty;
            switch (dbtype)
            {
                case DbType.Byte:
                case DbType.Currency:
                case DbType.Decimal:
                case DbType.Int16:
                case DbType.Int32:
                case DbType.Int64:
                    if (column.AllowDBNull)
                    {
                        result = "if(" + GetModuleInstanceName(column.Table) + "." + column.Name + ".ToString()!=null&&" + GetModuleInstanceName(column.Table) + "." + column.Name + ">0){db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + ");}";
                    }
                    else
                    {
                        result = "db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + ");";
                    }
                    break;
                case DbType.AnsiStringFixedLength:
                case DbType.AnsiString:
                case DbType.String:
                case DbType.StringFixedLength:
                case DbType.Binary:
                    if (column.AllowDBNull)
                    {
                        result = "if(" + GetModuleInstanceName(column.Table) + "." + column.Name + "!=null&&" + GetModuleInstanceName(column.Table) + "." + column.Name + ".Length>0){db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + "," + column.Size + ");}";
                    }
                    else
                    {
                        result = "db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + "," + column.Size + ");";
                    }
                    break;
                case DbType.Guid:
                    if (column.AllowDBNull)
                    {
                        result = "if(" + GetModuleInstanceName(column.Table) + "." + column.Name + ".ToString()!=null){db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + ");}";
                    }
                    else
                    {
                        result = "db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + ");";
                    }
                    break;
                case DbType.DateTime:
                case DbType.Date:
                    if (column.AllowDBNull)
                    {
                        result = "if(" + GetModuleInstanceName(column.Table) + "." + column.Name + "!=deadDate){db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + ");}";
                    }
                    else
                    {
                        result = "db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + ");";
                    }
                    break;
                default:
                    result = "db.AddParameter(\"@" + column.Name + "\"," + GetModuleInstanceName(column.Table) + "." + column.Name + ");";
                    break;
            }
            return result;
        }

        public string GetPrimaryKeyCSharpType()
        {
            return GetCSharpVariableType(SourceTable.PrimaryKey.MemberColumns[0]);
        }
        #endregion

        #region طĴ

        public string GetConstructorParameters()
        {
            string ret = "";
            foreach (ColumnSchema column in SourceTable.Columns)
            {
                ret += GetCSharpVariableType(column) + " " + GetCamelCaseName(GetPropertyName(column)) + ",\n\t\t\t";
            }
            return ret.Substring(0, ret.Length - 5);
        }

        public string GetAssignValue()
        {
            string ret = "";
            foreach (ColumnSchema column in SourceTable.Columns)
            {
                ret += "this." + GetMemberVariableName(column) + (new String(' ', GetPropertyNameMaxLength() - GetPropertyNameLength(column))) + " = " + GetCamelCaseName(GetPropertyName(column)) + ";\n\t\t\t";
            }
            return ret;
        }        

        public int GetPropertyNameLength(ColumnSchema column)
        {
            return (GetPropertyName(column)).Length;
        }
        public int GetPropertyNameMaxLength()
        {
            int ret = 0;
            foreach (ColumnSchema column in SourceTable.Columns)
            {
                ret = ret < GetPropertyNameLength(column) ? GetPropertyNameLength(column) : ret;
            }
            return ret;
        }        
        
        public override string GetFileName()
        {
            return this.GetClassName(this.SourceTable) + ".cs";
        }

        #endregion

        #region ԼӵĽű
        public string GetDataGridViewColumnType(ColumnSchema column)
        {
            switch (column.DataType)
            {
                case DbType.Boolean: return "DataGridViewCheckBoxColumn";
                default:
                    {
                        return "DataGridViewTextBoxColumn";
                    }
            }
        }
        #endregion
    }
}
